@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.56 2004/07/19 20:42:47 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.59 2004/09/23 16:01:49 sampie Exp $
  *
  *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -175,7 +175,7 @@ enum trig_func { SIN=1, COS=2, TAN=4 };
 #define MAXLINE        256        /* Length of character strings. */
 #endif /*MAXLINE*/
 
-#define MAXACC         10         /* Max. number of digits+1 after numeric point. */
+#define MAXACC         30    /* Max. number of digits after numeric point. */
 #define MAXBASES       4          /* Maximum number of numeric bases. */
 #define MAXCONFUN      10         /* Maximum number of constants/functions. */
 #define MAXDISPMODES   3          /* Maximum number of display modes. */
@@ -410,7 +410,7 @@ void do_delete();
 void do_numtype(enum num_type);
 void do_expno();
 void do_immed();
-void do_memory(int);
+void do_memory();
 void do_mode();
 void do_none();
 void do_number();
@@ -425,7 +425,6 @@ void exp_del();
 void get_constant(int);
 void get_display();
 void get_function(int);
-void get_key_val(int *, char *);
 void get_options(int, char **);
 void grey_buttons(enum base_type);
 void handle_menu_selection(struct button *, int);
@@ -446,7 +445,6 @@ void mperr();
 void MPstr_to_num(char *, enum base_type, int *);
 void paren_disp(char);
 void process_item(struct button *);
-void process_stack(int, int, int);
 void process_str(char *);
 void push_num(int *);
 void push_op(int);
@@ -457,8 +455,11 @@ void read_str(char **, char *);
 void refresh_display();
 void save_pending_values(struct button *);
 void save_resources();
+void set_accuracy_menu_item(int);
 void set_accuracy_toggle(int);
+void set_accuracy_tooltip(int);
 void set_display(char *, int);
+void write_display(char *);
 void set_error_state(int);
 void set_hyp_item(int);
 void set_ins_key();
@@ -490,7 +491,6 @@ void mpasin(int *, int *);
 void mpatan(int *, int *);
 void mpcdm(double *, int *);
 void mpcim(int *, int *);
-void mpclr(int *, int *);
 void mpcmd(int *, double *);
 void mpcmf(int *, int *);
 void mpcmi(int *, int *);
