@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.50 2004/07/23 11:22:16 sampie Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.54 2004/09/17 20:31:11 richb Exp $
  *
  *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -50,6 +50,8 @@ do_accuracy()     /* Set display accuracy. */
             v->accuracy = char_val(v->current->value[0]);
             SPRINTF(intval, "%d", v->accuracy);
             put_resource(R_ACCURACY, intval);
+            set_accuracy_menu_item(v->accuracy);
+            set_accuracy_tooltip(v->accuracy);
             make_registers();
             return;
         }
@@ -97,74 +99,7 @@ do_business()     /* Perform special business mode calculations. */
 }
 
 
-char* 
-ch_trig(char *func) {
-    assert(func);
-  
-    struct ch {
-        char *orig;
-        char *conv;
-    }; 
-    struct ch inv[] = {
-        { "Sin", "Asin"},
-        { "Cos", "Acos"},
-        { "Tan", "Atan"},
-        { NULL,  NULL}
-    };
-    struct ch hyp[] = {
-        { "Sin", "Sinh"},
-        { "Cos", "Cosh"},
-        { "Tan", "Tanh"},
-        { NULL,  NULL}
-    };
-    struct ch invhyp[] = {
-        { "Sin", "Asinh"},
-        { "Cos", "Acosh"},
-        { "Tan", "Atanh"},
-        { NULL,  NULL}
-    };
-  
-    int f1 = (v->inverse) ? 1 : 0;
-    int f2 = (v->hyperbolic) ? 2 : 0;
-    int table = f1 | f2;
-  
-    struct ch *ch = NULL;
-
-    switch (table) {
-      case 0:
-          break;
-
-      case 1:
-          ch = inv;
-          break;
-
-      case 2:
-        ch = hyp;
-        break;
-
-      case 3:
-        ch = invhyp;
-        break;
-
-      default:
-        assert(0);
-    }
-
-    if (ch) {
-        int i;
-
-        for (i = 0; ch[i].orig; i++) {
-            if (!strcmp(ch[i].orig, func)) {
-	        return(ch[i].conv);
-            }
-        }
-    } 
-
-    return(func);
-}
-
-
-char*
+char *
 gc_strdup(char *str)
 {
     char *dup;
@@ -217,7 +152,7 @@ exp_del()
 }
 
 
-int 
+static int 
 usable_num(int MPnum[MP_SIZE]) 
 {
     int ret = 0;
@@ -232,7 +167,7 @@ usable_num(int MPnum[MP_SIZE])
 }
 
 
-void
+static void
 exp_del_char(char **expr, int amount) 
 {
     char *e = NULL;
@@ -257,7 +192,7 @@ exp_del_char(char **expr, int amount)
 }
 
 
-void
+static void
 exp_replace(char *text)
 {
     free(v->expression);
@@ -266,7 +201,7 @@ exp_replace(char *text)
 }
 
 
-void
+static void
 exp_negate()
 {
     if (v->expression) {
@@ -284,7 +219,7 @@ exp_negate()
 }
 
 
-void
+static void
 exp_inv()
 {
     if (v->expression) {
@@ -302,7 +237,7 @@ exp_inv()
 }
 
 
-int
+static int
 exp_has_postfix(char *str, char *postfix)
 {
     int len, plen;
@@ -369,55 +304,6 @@ str_replace(char **str, char *from, char *to)
 }
 
 
-void
-exp_do_backspace()
-{
-    if (exp_has_postfix(v->expression, "Ans")) { 
-        char *ans = make_number(v->e.ans, v->base, TRUE, FALSE);   
-
-        str_replace(&v->expression, "Ans", ans);
-    }
-}
-
-
-char *
-exp_print()
-{
-    int i, len;
-
-    if (!v->expression) return NULL;
-
-    i = 0;
-    len = strlen(v->expression);
-
-    for (i = 0; len-i >= 3; i++) {
-        if (!strncasecmp("ans", v->expression+i, 3)) {
-            char *ans;
-            int j = i+3;
-            char *prefix = malloc(i+1);
-            char *postfix = malloc(len-j+1);
-
-            assert(prefix && postfix);
-            memset(prefix, 0, i+1);
-            memset(postfix, 0, len-j+1);
-            memcpy(prefix, v->expression, i);
-            memcpy(postfix, v->expression+i+3, len-j);
-
-            ans = make_number(v->e.ans, v->base, TRUE, FALSE);
-            assert(ans);
-            char *print = malloc(strlen(ans)+i+len-j+1);
-            sprintf(print, "%s%s%s", prefix, ans, postfix);
-            free(prefix);
-            free(postfix);
-
-            return(print);
-        }
-    }
-
-    return(NULL);
-}
-
-
 void
 do_expression()
 {
@@ -633,7 +519,7 @@ do_calc()      /* Perform arithmetic calculation and display result. */
 }
 
 
-int
+static int
 do_trigfunc(int s[MP_SIZE], int t[MP_SIZE])
 {
     enum trig_func tfunc;
@@ -671,8 +557,6 @@ do_trig()
 int
 do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc)
 {
-    if (!v->current) return -EINVAL;
-
     enum mode {
         normal = 0,
         inv = 1,
@@ -680,8 +564,15 @@ do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc)
         invhyp = 3,
     } mode;
 
-    int inverse = (v->inverse) ? inv : 0; 
-    int hyperbolic = (v->hyperbolic) ? hyp : 0; 
+    int inverse;
+    int hyperbolic;
+
+    if (!v->current) {
+        return(-EINVAL);
+    }
+
+    inverse = (v->inverse) ? inv : 0;
+    hyperbolic = (v->hyperbolic) ? hyp : 0;
 
     mode = (inverse | hyperbolic);
 
@@ -929,7 +820,7 @@ do_function()      /* Perform a user defined function. */
 }
 
 
-void
+static void
 do_immedfunc(int s[MP_SIZE], int t[MP_SIZE])
 {
     int MP1[MP_SIZE];
@@ -978,7 +869,7 @@ do_immed()
 
 
 void
-do_memory(int show)
+do_memory()
 {
     make_registers();
     put_resource(R_REGS, set_bool(v->rstate == TRUE));
@@ -1123,6 +1014,8 @@ do_sto()
         default:
             assert(0);
     }
+
+    make_registers();
 }
 
 
@@ -1182,7 +1075,7 @@ do_rcl_reg(int reg, int value[MP_SIZE])
 }
 
 
-void
+static void
 syntaxdep_show_display()
 {
     switch (v->syntax) {
@@ -1284,7 +1177,7 @@ do_point()                   /* Handle numeric point. */
 }
 
 
-void
+static void
 do_portionfunc(int num[MP_SIZE])
 {
     int MP1[MP_SIZE];
@@ -1356,7 +1249,6 @@ do_shift()     /* Perform bitwise shift on display value. */
         case exprs: {
             enum shiftd dir;
             int MPval[MP_SIZE];
-            int MPres[MP_SIZE];
             int n = char_val(v->current->value[0]);
             int ret = usable_num(MPval);
 
@@ -1395,7 +1287,7 @@ do_trigtype(enum trig_type t)    /* Change the current trigonometric type. */
 }
 
 
-int
+static int
 key_equal(struct button *x, struct button y)
 {
     return(x->value[0] == y.value[0] && x->mods[0] == y.mods[0]);
@@ -1447,7 +1339,7 @@ save_pending_values(struct button *but)
 }
 
 
-struct button *
+static struct button *
 ch_to_button(struct button buttons[], int max_buttons, char ch)
 {
     int i;
@@ -1462,7 +1354,7 @@ ch_to_button(struct button buttons[], int max_buttons, char ch)
 }
 
 
-struct button *
+static struct button *
 val_to_button(struct button buttons[], int max_buttons, int val)
 {
     int i;
