@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.126.2.3 2004/09/17 14:41:28 danilo Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.139 2004/09/23 16:01:49 sampie Exp $
  *
- *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  */
-
 #include "config.h"
 
 #include <stdio.h>
@@ -32,10 +31,13 @@
 #include "calctool.h"
 #include "extern.h"
 #include "dsdefs.h"
+#include "functions.h"
+#include "lr_parser.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <gconf/gconf-client.h>
+#include "ce_parser.h"
 
 #define BUT_0     X->bas_buttons[24]       /* 0 */
 #define BUT_1     X->bas_buttons[16]       /* 1 */
@@ -48,13 +50,13 @@
 #define BUT_8     X->bas_buttons[1]        /* 8 */
 #define BUT_9     X->bas_buttons[2]        /* 9 */
 #define BUT_CLR   X->bas_buttons[7]        /* Clr */
-#define BUT_ACC   X->sci_buttons[7]        /* A */
-#define BUT_A     X->sci_buttons[24]       /* a */
-#define BUT_B     X->sci_buttons[25]       /* b */
-#define BUT_C     X->sci_buttons[26]       /* c */
-#define BUT_D     X->sci_buttons[16]       /* d */
-#define BUT_E     X->sci_buttons[17]       /* e */
-#define BUT_F     X->sci_buttons[18]       /* f */
+#define BUT_ACC   X->sci_buttons[7]        /* a */
+#define BUT_A     X->sci_buttons[24]       /* A */
+#define BUT_B     X->sci_buttons[25]       /* B */
+#define BUT_C     X->sci_buttons[26]       /* C */
+#define BUT_D     X->sci_buttons[16]       /* D */
+#define BUT_E     X->sci_buttons[17]       /* E */
+#define BUT_F     X->sci_buttons[18]       /* F */
 
 struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkAccelGroup *kbd_accel;
@@ -77,8 +79,15 @@ struct Xobject {               /* Gtk+/Xlib graphics object. */
     GdkPixbuf *icon;                   /* Main window icon. */
     GtkWidget *menubar;
     GtkWidget *mode_panel;
+
+    GtkWidget *khbox;                  /* Box containing statusbar and image */
+    GtkWidget *status_image;           /* Statusbar image */
+    GtkWidget *statusbar; 
+
     GtkWidget *display_item;           /* Calculator display. */
     GtkWidget *rframe;                 /* Register window. */
+    GtkWidget *spframe;                /* Set Precision window. */
+    GtkWidget *spframe_val;
     GtkWidget *regs[MAXREGS];          /* Memory registers. */
     GtkWidget *menus[MAXMENUS];
     GtkWidget *trig[MAXTRIGMODES];     /* Trigonometric mode. */
@@ -128,7 +137,9 @@ static char *make_hostname(Display *);
 static gboolean aframe_key_cb(GtkWidget *, GdkEventKey *, gpointer);
 static gboolean dismiss_aframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean dismiss_rframe(GtkWidget *, GdkEvent *, gpointer);
+static gboolean dismiss_spframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean kframe_key_press_cb(GtkWidget *, GdkEventKey *, gpointer);
+static gboolean spframe_key_cb(GtkWidget *, GdkEventKey *, gpointer);
 
 static void about_cb(GtkAction *action);
 static void add_cf_column(GtkTreeView *, gchar *, gint, gboolean);
@@ -160,12 +171,15 @@ static void set_gcalctool_icon(void);
 static void set_memory_toggle(int);
 static void set_show_tsep_toggle(int);
 static void set_show_zeroes_toggle(int);
+static void show_precision_frame();
+static void spframe_cancel_cb(GtkButton *, gpointer);
+static void spframe_ok_cb(GtkButton *, gpointer);
 static void trig_cb(GtkToggleButton *, gpointer);
 static void ts_proc(GtkAction *action);
-static void gcalc_window_have_icons_notify(GConfClient *client, guint cnxn_id,
-                                           GConfEntry *entry, gpointer data);
-static void gcalc_window_get_menu_items(XVars X);
-static void gcalc_window_set_unset_image(gboolean have_icons);
+static void gcalc_window_have_icons_notify(GConfClient *, guint,
+                                            GConfEntry *, gpointer);
+static void gcalc_window_get_menu_items(XVars);
+static void gcalc_window_set_unset_image(gboolean);
 
 #define MENU_KEY_DIR "/desktop/gnome/interface"
 
@@ -266,33 +280,36 @@ static GtkToggleActionEntry toggle_entries[] = {
       N_("Show thousands separator"), G_CALLBACK(ts_proc),   FALSE },
     { "Memory",    NULL, N_("_Memory Registers"),         "<control>M",
       N_("Show memory registers"),    G_CALLBACK(mb_proc),   FALSE },
-
     { "Show",      NULL, N_("Show _Trailing Zeroes"),     "<control>T",
       N_("Show trailing zeroes"),     G_CALLBACK(astz_proc), FALSE },
+    { "ArithmeticPrecedence", NULL, N_("_Use Arithmetic Precedence"), "<control>U",
+      N_("Use Arithmetic Precedence"), G_CALLBACK(mb_proc), FALSE },
 };
 static guint n_toggle_entries = G_N_ELEMENTS(toggle_entries);
 
 static GtkRadioActionEntry acc_radio_entries[] = {
-  { "SP0", NULL, N_("0 significant places"), NULL, 
+  { "SP0", NULL, N_("_0 significant places"), "<control>0", 
     N_("0 significant places"), '0' },
-  { "SP1", NULL, N_("1 significant place"),  NULL, 
+  { "SP1", NULL, N_("_1 significant place"),  "<control>1", 
     N_("1 significant place"),  '1' },
-  { "SP2", NULL, N_("2 significant places"), NULL,
+  { "SP2", NULL, N_("_2 significant places"), "<control>2",
     N_("2 significant places"), '2' },
-  { "SP3", NULL, N_("3 significant places"), NULL,
+  { "SP3", NULL, N_("_3 significant places"), "<control>3",
     N_("3 significant places"), '3' },
-  { "SP4", NULL, N_("4 significant places"), NULL,
+  { "SP4", NULL, N_("_4 significant places"), "<control>4",
     N_("4 significant places"), '4' },
-  { "SP5", NULL, N_("5 significant places"), NULL,
+  { "SP5", NULL, N_("_5 significant places"), "<control>5",
     N_("5 significant places"), '5' },
-  { "SP6", NULL, N_("6 significant places"), NULL,
+  { "SP6", NULL, N_("_6 significant places"), "<control>6",
     N_("6 significant places"), '6' },
-  { "SP7", NULL, N_("7 significant places"), NULL,
+  { "SP7", NULL, N_("_7 significant places"), "<control>7",
     N_("7 significant places"), '7' },
-  { "SP8", NULL, N_("8 significant places"), NULL,
+  { "SP8", NULL, N_("_8 significant places"), "<control>8",
     N_("8 significant places"), '8' },
-  { "SP9", NULL, N_("9 significant places"), NULL,
+  { "SP9", NULL, N_("_9 significant places"), "<control>9",
     N_("9 significant places"), '9' },
+  { "SPOther", NULL, N_("_Other (10) ..."), "<control>O",
+    N_("Set other precision"), 'O' },
 };
 static guint n_acc_radio_entries = G_N_ELEMENTS(acc_radio_entries);
 
@@ -304,8 +321,8 @@ static GtkRadioActionEntry base_radio_entries[] = {
   { "Scientific", NULL, N_("_Scientific mode"), "<control>S",
     N_("Scientific mode"), M_SCI },
 };
-static guint n_base_radio_entries = G_N_ELEMENTS(base_radio_entries);
 
+static guint n_base_radio_entries = G_N_ELEMENTS(base_radio_entries);
 
 static const gchar *ui_info =
 "<ui>"
@@ -328,6 +345,8 @@ static const gchar *ui_info =
 "      <menuitem action='Thousands'/>"
 "      <separator/>"
 "      <menuitem action='Memory'/>"
+"      <separator/>"
+"      <menuitem action='ArithmeticPrecedence'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='Contents'/>"
@@ -345,6 +364,7 @@ static const gchar *ui_info =
 "    <menuitem action='SP7'/>"
 "    <menuitem action='SP8'/>"
 "    <menuitem action='SP9'/>"
+"    <menuitem action='SPOther'/>"
 "    <separator/>"
 "    <menuitem action='Show'/>"
 "  </popup>"
@@ -425,11 +445,25 @@ main(int argc, char **argv)
     do_calctool(argc, argv);
 
     return(0);
-
-/*NOTREACHED*/
 }
 
 
+void 
+update_statusbar(gchar *text, const gchar *imagename)
+{
+    GtkImage *image = GTK_IMAGE(X->status_image);
+
+    assert(text);
+    assert(imagename);
+    assert(image);
+
+    gtk_image_set_from_stock(image, imagename, GTK_ICON_SIZE_BUTTON);
+    gtk_statusbar_pop(GTK_STATUSBAR(X->statusbar), 0);
+    gtk_statusbar_push(GTK_STATUSBAR(X->statusbar), 0, text); 
+}
+
+
+/*ARGSUSED*/
 static void
 about_cb(GtkAction *action)
 {
@@ -438,6 +472,7 @@ about_cb(GtkAction *action)
     if (about == NULL) {
         const gchar *authors[] = {
             "Rich Burridge <rich.burridge@sun.com>",
+            "Sami Pietila <sampie@ariana-dsl.utu.fi>",
             NULL
         };
         const gchar *documenters[] = {
@@ -446,8 +481,9 @@ about_cb(GtkAction *action)
         };
         const gchar *translator_credits = _("translator_credits");
 
-        about = gnome_about_new(_("Gcalctool"), VERSION,
-                       "(C) 2003 the Free Software Foundation",
+        about = gnome_about_new(_("Gcalctool"),
+                       VERSION,
+                       "(C) 2004 the Free Software Foundation",
                        _("Calculator with financial and scientific modes."),
                        authors,
                        documenters,
@@ -490,7 +526,6 @@ add_cf_column(GtkTreeView *treeview, gchar *name, gint colno, gboolean editable)
     }
 }
 
-
 /*ARGSUSED*/
 static void
 aframe_cancel_cb(GtkButton *button, gpointer user_data)
@@ -499,6 +534,7 @@ aframe_cancel_cb(GtkButton *button, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
 static gboolean
 aframe_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
@@ -527,6 +563,7 @@ aframe_ok_cb(GtkButton *button, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
 static void
 astz_proc(GtkAction *action)
 {
@@ -546,6 +583,7 @@ astz_proc(GtkAction *action)
 }
 
 
+/*ARGSUSED*/
 static void
 base_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -560,50 +598,6 @@ beep()
 }
 
 
-static GtkWidget *
-button_new_with_stock_image(const gchar *text, const gchar *stock_id)
-{
-    GtkWidget *button, *label, *image, *hbox, *align;
-    GtkStockItem item;
-
-    button = gtk_button_new();
-
-    if (GTK_BIN(button)->child) {
-        gtk_container_remove(GTK_CONTAINER(button), GTK_BIN(button)->child);
-    }
-
-    if (gtk_stock_lookup(stock_id, &item)) {
-        label = gtk_label_new_with_mnemonic(text);
-
-        gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(button));
-
-        image = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
-        hbox = gtk_hbox_new(FALSE, 2);
-
-        align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-
-        gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-        gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-        gtk_container_add(GTK_CONTAINER(button), align);
-        gtk_container_add(GTK_CONTAINER(align), hbox);
-        gtk_widget_show_all(align);
-
-        return(button);
-    }
-
-    label = gtk_label_new_with_mnemonic(text);
-    gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(button));
-
-    gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-
-    gtk_widget_show(label);
-    gtk_container_add(GTK_CONTAINER(button), label);
-
-    return(button);
-}
-
-
 /*ARGSUSED*/
 static void
 button_proc(GtkButton *widget, gpointer user_data)
@@ -611,19 +605,31 @@ button_proc(GtkButton *widget, gpointer user_data)
     struct button *n;
 
     n = (struct button *) g_object_get_data(G_OBJECT(widget), "button");
-    if (v->pending) {
-        if (v->current != NULL) {
-            free(v->current);
-        }
-        v->current = copy_button_info(n);
-        do_pending();
-    } else {
-        process_item(n);
-    }
 
-    if (v->new_input && v->dtype == FIX) {
-        STRCPY(v->fnum, v->display);
-        set_display(v->fnum, FALSE);
+    switch (v->syntax) {
+        case npa:
+            if (v->pending) {
+                if (v->current != NULL) {
+	            free(v->current);
+                }
+                v->current = copy_button_info(n);
+                do_pending();
+            } else {
+                process_item(n);
+            }
+            if (v->new_input && v->dtype == FIX) {
+                STRCPY(v->fnum, v->display);
+                set_display(v->fnum, FALSE);
+            }
+            break;
+
+        case exprs:
+            v->current = copy_button_info(n);
+            do_expression();
+            break;
+
+        default:
+            assert(0);
     }
 }
 
@@ -671,6 +677,7 @@ cell_edited(GtkCellRendererText *cell, const gchar *path_string,
 }
 
 
+/*ARGSUSED*/
 static void
 cfframe_response_cb(GtkDialog *dialog, gint id, gpointer data)
 {
@@ -932,35 +939,96 @@ create_cfframe(enum menu_type mtype, GtkWidget *dialog)
 }
 
 
+static void
+create_spframe()     /* Create auxiliary frame for Set Precision value. */
+{
+    GtkWidget *vbox, *hbox, *button_hbox, *label;
+    GtkWidget *set_button, *cancel_button;
+
+    X->spframe = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_title(GTK_WINDOW(X->spframe), _("Set Precision"));
+    gtk_window_set_resizable(GTK_WINDOW(X->spframe), FALSE);
+    gtk_window_set_transient_for(GTK_WINDOW(X->spframe), GTK_WINDOW(X->kframe));
+    gtk_window_set_type_hint(GTK_WINDOW(X->spframe),
+                             GDK_WINDOW_TYPE_HINT_DIALOG);
+
+    vbox = gtk_vbox_new(FALSE, 12);
+    gtk_widget_show(vbox);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
+    gtk_container_add(GTK_CONTAINER(X->spframe), vbox);
+
+    hbox = gtk_hbox_new(FALSE, 6);
+    gtk_widget_show(hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+    label = gtk_label_new_with_mnemonic(_("Significant _places:"));
+    gtk_widget_show(label);
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
+
+    X->spframe_val = gtk_spin_button_new_with_range(0, MAXACC, 1);
+    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(X->spframe_val),
+                                      GTK_UPDATE_IF_VALID);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(X->spframe_val), 
+                              (double) v->accuracy);
+    gtk_widget_show(X->spframe_val);
+    gtk_box_pack_start(GTK_BOX(hbox), X->spframe_val, FALSE, FALSE, 0);
+
+    button_hbox = gtk_hbutton_box_new();
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(button_hbox), GTK_BUTTONBOX_END);
+    gtk_widget_ref(button_hbox);
+    gtk_widget_show(button_hbox);
+    gtk_box_pack_start(GTK_BOX(vbox), button_hbox, TRUE, TRUE, 0);
+    gtk_box_set_spacing(GTK_BOX(button_hbox), 12);
+
+    cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
+    gtk_widget_ref(cancel_button);
+    gtk_widget_show(cancel_button);
+    gtk_box_pack_start(GTK_BOX(button_hbox), cancel_button, FALSE, FALSE, 0);
+
+    set_button = gtk_button_new_with_mnemonic(_("_Set"));
+    g_return_if_fail(set_button != NULL);
+    gtk_widget_ref(set_button);
+    gtk_widget_show(set_button);
+    GTK_WIDGET_SET_FLAGS(set_button, GTK_CAN_DEFAULT);
+    gtk_window_set_default(GTK_WINDOW(X->spframe), set_button);
+    gtk_box_pack_start(GTK_BOX(button_hbox), set_button, FALSE, FALSE, 0);
+
+    g_signal_connect(G_OBJECT(X->spframe), "delete_event",
+                     G_CALLBACK(dismiss_spframe), NULL);
+    g_signal_connect(G_OBJECT(X->spframe), "key_press_event",
+                     G_CALLBACK(spframe_key_cb), NULL);
+    g_signal_connect(G_OBJECT(set_button), "clicked",
+                     G_CALLBACK(spframe_ok_cb), NULL);
+    g_signal_connect(G_OBJECT(cancel_button), "clicked",
+                     G_CALLBACK(spframe_cancel_cb), NULL);
+    gtk_widget_realize(X->spframe);
+}
+
+
+/* action_image[] is a list of Actions which are having icons associated.
+ * We need to set/unset the icons if /desktop/gnome/interface/menus_have_icons
+ * toggles.
+ */
+
 static void
 gcalc_window_get_menu_items(XVars X)
 {
     GtkAction *act;
     GSList *p = NULL;
     gint i = 0;
-
-/* action_image[] is a list of Actions which are having icons associated.
- * We need to set/unset the icons if 
- * /desktop/gnome/interface/menus_have_icons toggles.
- */
-
-    static gchar *action_image[] = {
-        "Quit", 
-        "Copy", 
-        "Paste", 
-        "Contents", 
-        "About", 
-        NULL
-    };
+    static gchar *action_image[] = { "Quit", "Copy", "Paste", 
+                                     "Contents", "About", NULL };
 
     while (action_image[i]) {
-        act = gtk_action_group_get_action(X->actions, action_image[i]);
-        p = gtk_action_get_proxies(GTK_ACTION(act));
+        act = gtk_action_group_get_action (X->actions, action_image[i]);
+        p = gtk_action_get_proxies (GTK_ACTION (act));
 
         for (; p; p = p->next) {
             if (p->data) {
-                if (GTK_IS_MENU_ITEM(p->data)) {
-                    list = g_slist_append(list, p->data);
+                if (GTK_IS_MENU_ITEM (p->data)) {
+                    list = g_slist_append (list, p->data);
                 }
             }
         }
@@ -970,6 +1038,7 @@ gcalc_window_get_menu_items(XVars X)
 }
 
 
+/*ARGSUSED*/
 static void
 gcalc_window_have_icons_notify(GConfClient *client, guint cnxn_id,
                                GConfEntry *entry, gpointer data)
@@ -990,7 +1059,7 @@ gcalc_window_set_unset_image(gboolean have_icons)
 
         if (image && ! g_object_get_data(G_OBJECT(temp->data), "saved_image")) {
             g_object_set_data_full(G_OBJECT(temp->data), "saved_image", 
-                                   g_object_ref (image), g_object_unref);
+                                   g_object_ref(image), g_object_unref);
         }
 
         if (!image) {
@@ -1010,14 +1079,14 @@ gcalc_window_set_unset_image(gboolean have_icons)
 }
 
 
-void
+static void
 create_kframe()
 {
     char *hn;
-    int count;
     GError *error;
     GtkWidget *event_box, *view_widget;
     GtkTextBuffer *buffer;
+    GtkWidget *scrolledwindow;
 
     v->tool_label = NULL;
     if (v->titleline == NULL) {
@@ -1043,7 +1112,7 @@ create_kframe()
     gtk_widget_show(X->kvbox);
 
     X->actions = gtk_action_group_new("Actions");
-    gtk_action_group_set_translation_domain(X->actions, GETTEXT_PACKAGE);
+    gtk_action_group_set_translation_domain (X->actions, NULL);
     gtk_action_group_add_actions(X->actions, entries, n_entries, NULL);
     gtk_action_group_add_toggle_actions(X->actions,
                                         toggle_entries, n_toggle_entries,
@@ -1071,13 +1140,21 @@ create_kframe()
     gtk_widget_show(X->menubar);
     gtk_box_pack_start(GTK_BOX(X->kvbox), X->menubar, FALSE, FALSE, 0);
 
+
+    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
+    gtk_widget_show(scrolledwindow);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwindow), 
+				   GTK_POLICY_ALWAYS, 
+				   GTK_POLICY_NEVER);
+
+
     event_box = gtk_event_box_new();
     X->display_item = gtk_text_view_new();
     gtk_widget_set_name(X->display_item, "displayitem");
 
     gtk_text_view_set_justification(GTK_TEXT_VIEW(X->display_item),
                                     GTK_JUSTIFY_RIGHT);
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(X->display_item), GTK_WRAP_WORD);
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(X->display_item), GTK_WRAP_NONE);
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(X->display_item));
     gtk_text_buffer_create_tag(buffer, "x-large", "scale", PANGO_SCALE_X_LARGE, 
                                NULL);			       
@@ -1086,16 +1163,18 @@ create_kframe()
     gtk_text_view_set_editable(GTK_TEXT_VIEW(X->display_item), FALSE);
     GTK_WIDGET_SET_FLAGS(X->display_item, GTK_CAN_FOCUS);
 
-    gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(X->display_item), 12);
-    gtk_text_view_set_pixels_below_lines(GTK_TEXT_VIEW(X->display_item), 12);
+    gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(X->display_item), 8);
+    gtk_text_view_set_pixels_below_lines(GTK_TEXT_VIEW(X->display_item), 8);
     gtk_text_view_set_right_margin(GTK_TEXT_VIEW(X->display_item), 6);
 
+    gtk_container_add(GTK_CONTAINER(scrolledwindow), X->display_item);
     atk_object_set_role(gtk_widget_get_accessible(X->display_item), 
-                        ATK_ROLE_EDITBAR);
+                                                  ATK_ROLE_EDITBAR);
     set_display("0.00", FALSE);
+
     gtk_widget_ref(X->display_item);
     gtk_container_set_border_width(GTK_CONTAINER(X->display_item), 2);
-    gtk_container_add(GTK_CONTAINER(event_box), X->display_item);
+    gtk_container_add(GTK_CONTAINER(event_box), scrolledwindow);
     gtk_widget_show(X->display_item);
     gtk_box_pack_start(GTK_BOX(X->kvbox), event_box, FALSE, TRUE, 0);
     gtk_widget_show(event_box);
@@ -1110,6 +1189,7 @@ create_kframe()
 
     X->sci_panel = make_but_panel(X->kvbox, X->sci_buttons,
                                   s_buttons, SROWS, SCOLS, "sci");
+    set_accuracy_tooltip(v->accuracy);
 
     gtk_widget_show(X->fin_panel);
     gtk_widget_show(X->mode_panel);
@@ -1123,6 +1203,18 @@ create_kframe()
     gtk_window_set_icon(GTK_WINDOW(X->kframe), X->icon);
     gtk_window_set_focus(GTK_WINDOW(X->kframe), GTK_WIDGET(BUT_CLR));
 
+    X->khbox = gtk_hbox_new(FALSE, 0);
+    gtk_widget_show(X->khbox);
+    gtk_container_add(GTK_CONTAINER(X->kvbox), X->khbox);
+    X->statusbar = gtk_statusbar_new();
+    gtk_widget_show(X->statusbar);
+    gtk_box_pack_start(GTK_BOX(X->khbox), X->statusbar, TRUE, TRUE, 0);
+    gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(X->statusbar), FALSE);
+
+    X->status_image = gtk_image_new_from_stock("", 
+					       GTK_ICON_SIZE_BUTTON);
+    gtk_widget_show(X->status_image);
+    gtk_box_pack_start(GTK_BOX(X->khbox), X->status_image, FALSE, TRUE, 0);
     g_signal_connect(G_OBJECT(X->kframe), "key_press_event",
                      G_CALLBACK(kframe_key_press_cb), NULL);
 
@@ -1131,30 +1223,36 @@ create_kframe()
             view_widget = gtk_ui_manager_get_widget(X->ui, 
                                               "/MenuBar/ViewMenu/Financial");
             break;
+
         case SCIENTIFIC:
             view_widget = gtk_ui_manager_get_widget(X->ui, 
                                               "/MenuBar/ViewMenu/Scientific");
             break;
+
         default:
             view_widget = gtk_ui_manager_get_widget(X->ui, 
                                               "/MenuBar/ViewMenu/Basic");
             break;
     }
+
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(view_widget), TRUE);
+    gcalc_window_get_menu_items(X);
 
-    gcalc_window_get_menu_items (X);
-
-    gconf_client_add_dir (X->client,
-                          MENU_KEY_DIR,
-                          GCONF_CLIENT_PRELOAD_NONE,
-                          NULL);
-    gconf_client_notify_add (X->client,
-                             MENU_KEY_DIR"/menus_have_icons",
-                             (GConfClientNotifyFunc)
-                               gcalc_window_have_icons_notify,
+    gconf_client_add_dir(X->client, MENU_KEY_DIR,
+                         GCONF_CLIENT_PRELOAD_NONE, NULL);
+    gconf_client_notify_add(X->client, MENU_KEY_DIR"/menus_have_icons",
+                            (GConfClientNotifyFunc) 
+                                gcalc_window_have_icons_notify,
                              NULL, NULL, NULL);
     gcalc_window_set_unset_image(gconf_client_get_bool(X->client, 
                                  MENU_KEY_DIR"/menus_have_icons", NULL));
+
+    /* Use loaded Arithmetic Precedence mode setting. */
+    if (v->syntax == exprs) {
+        view_widget = gtk_ui_manager_get_widget(X->ui, 
+                                    "/MenuBar/ViewMenu/ArithmeticPrecedence");
+        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(view_widget), TRUE);
+    }
 }
 
 
@@ -1284,7 +1382,7 @@ create_mode_panel(GtkWidget *main_vbox)
 }
 
 
-void
+static void
 create_rframe()
 {
     char line[MAXLINE];     /* Current memory register line. */
@@ -1321,6 +1419,7 @@ create_rframe()
 }
 
 
+/*ARGSUSED*/
 static void
 die_cb(GnomeClient *client, gpointer data)
 {
@@ -1350,6 +1449,16 @@ dismiss_rframe(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
+static gboolean
+dismiss_spframe(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+    X->spframe = NULL;
+
+    return(FALSE);
+}
+
+
 static void
 create_con_fun_menu(enum menu_type mtype)
 {
@@ -1378,8 +1487,8 @@ create_con_fun_menu(enum menu_type mtype)
             if (!strlen(v->fun_vals[i])) {
                 invalid = 1;
             } else {
-                SPRINTF(mline, "<span weight=\"bold\">%s%1d:</span> %s [%s]", 
-                        _("F"), i, v->fun_vals[i], v->fun_names[i]);
+	      SPRINTF(mline, "<span weight=\"bold\">%s%1d:</span> %s [%s]", 
+		      _("F"), i, v->fun_vals[i], v->fun_names[i]);
             }
         }
 
@@ -1411,6 +1520,7 @@ create_menu_item_with_markup(char *label, int menu_no, int user_data)
 }
 
 
+/*ARGSUSED*/
 static void
 disp_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -1439,7 +1549,7 @@ get_constant(int n)
 }
 
 
-void
+static void
 get_display()              /* The Copy function key has been pressed. */
 {
     gchar *string = NULL;
@@ -1489,12 +1599,9 @@ get_function(int n)
 static char *
 get_localized_numeric_point(void)
 {
-    struct lconv *locale_data;
     const char *decimal_point;
-    int decimal_point_len;
 
     decimal_point = localeconv()->decimal_point;
-    decimal_point_len = strlen(decimal_point);
 
     return(g_locale_to_utf8(decimal_point, -1, NULL, NULL, NULL));
 }
@@ -1523,7 +1630,27 @@ get_menu_entry(enum menu_type mtype, int offset)
 static void
 get_proc(GtkClipboard *clipboard, const gchar *text, gpointer data)
 {
-    process_str((char *) text);
+    switch (v->syntax) {
+        case npa: {
+            int ret = lr_parse((char *) text, v->MPdisp_val);
+
+            if (!ret) {
+                show_display(v->MPdisp_val);
+            } else {
+                update_statusbar(_("Clipboard contained malformed calculation"),
+                                 "gtk-dialog-error");
+            }
+            break;
+        }
+    
+        case exprs:
+            exp_append((char *) text);
+            refresh_display();
+            break;
+    
+        default:
+            assert(0);
+    }
 }
 
 
@@ -1563,7 +1690,7 @@ grey_buttons(enum base_type base)
 }
 
 
-void
+static void
 handle_selection()  /* Handle the GET function key being pressed. */
 {
     gtk_clipboard_request_text(gtk_clipboard_get(X->clipboard_atom),
@@ -1585,6 +1712,7 @@ help_cb()
 }
 
 
+/*ARGSUSED*/
 static void
 hyp_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -1592,6 +1720,7 @@ hyp_cb(GtkToggleButton *button, gpointer user_data)
 }
 
 
+/*ARGSUSED*/
 static void
 inv_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -1606,7 +1735,7 @@ check_for_localized_numeric_point(int keyval)
     gunichar ch;
 
     ch = gdk_keyval_to_unicode(keyval);
-    g_return_if_fail(g_unichar_validate(ch));
+    g_return_val_if_fail(g_unichar_validate(ch), FALSE);
 
     outbuf[g_unichar_to_utf8(ch, outbuf)] = '\0';
 
@@ -1637,11 +1766,10 @@ check_vals(int n, int keyval, int state,
     return(FALSE);
 }
 
-
+/*ARGSUSED*/
 static gboolean
 kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-    GtkWidget *focus;
     int retval = FALSE;
 
     if (check_for_localized_numeric_point(event->keyval) == TRUE) {
@@ -1674,10 +1802,9 @@ kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
             break;
     }
 
-    return(retval);
+    return retval;
 }
 
-
 void
 load_resources()        /* Load gconf configuration database for gcalctool. */
 { 
@@ -1693,7 +1820,6 @@ void
 make_frames()
 {
     struct button *n;
-    GtkWidget *menu;
 
     X->clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);
     create_kframe();                     /* Create main gcalctool window. */
@@ -1701,7 +1827,7 @@ make_frames()
     set_mode(v->modetype);
 
     n = (struct button *) g_object_get_data(G_OBJECT(BUT_ACC), "button");
-    menu = create_menu(n->mtype, n);
+    (void) create_menu(n->mtype, n);
     set_accuracy_toggle(v->accuracy);
     set_show_tsep_toggle(v->show_tsep);
     set_show_zeroes_toggle(v->show_zeroes);
@@ -1753,6 +1879,28 @@ set_accessible_name(GtkWidget *widget, struct button button)
 }
 
 
+void
+set_accuracy_menu_item(int accuracy)
+{
+    char label[MAXLINE];
+
+    SPRINTF(label, _("Other (%d) ..."), accuracy);
+    g_object_set(gtk_ui_manager_get_action(X->ui, "/AccMenu/SPOther"),
+                 "label", label, NULL);
+}
+
+
+void
+set_accuracy_tooltip(int accuracy)
+{
+    char tooltip[MAXLINE];
+
+    SPRINTF(tooltip, _("Set accuracy from 0 to %d numeric places. Currently set to %d places. [a]"), 
+            MAXACC, accuracy);
+    gtk_tooltips_set_tip(X->tips, BUT_ACC, tooltip, "");
+}
+
+
 static GtkWidget *
 make_but_panel(GtkWidget *vbox, GtkWidget **Gtk_buttons,
                struct button buttons[], int rows, int cols, char *tag)
@@ -1823,8 +1971,7 @@ make_reg(int n, char *str)
 static GtkWidget *
 create_menu(enum menu_type mtype, struct button *n)
 {
-    int count = 0;
-    int m     = (int) mtype;
+    int m = (int) mtype;
 
     switch (mtype) {
         case M_EXCH :
@@ -1845,6 +1992,7 @@ create_menu(enum menu_type mtype, struct button *n)
     if (X->menus[m] == NULL) {
         if (mtype == M_ACC) {
             X->menus[m] = gtk_ui_manager_get_widget(X->ui, "/AccMenu");
+            set_accuracy_menu_item(v->accuracy);
         } else if (mtype == M_LSHF) {
             X->menus[m] = gtk_ui_manager_get_widget(X->ui, "/LeftShiftMenu");
         } else if (mtype == M_RSHF) {
@@ -1854,6 +2002,7 @@ create_menu(enum menu_type mtype, struct button *n)
 
     gtk_container_set_border_width(GTK_CONTAINER(X->menus[m]), 1);
     X->mrec[m] = n;
+
     return(X->menus[m]);
 }
 
@@ -1865,6 +2014,7 @@ menu_proc_cb(GtkMenuItem *mi, gpointer user_data)
     int mtype = (int) g_object_get_data(G_OBJECT(mi), "mtype");
 
     v->current->value[0] = '0' + (int) user_data;
+    
     handle_menu_selection(X->mrec[mtype], v->current->value[0]);
 }
 
@@ -1888,6 +2038,7 @@ menu_button_button_press_cb(GtkWidget *widget,
 }
 
 
+/*ARGSUSED*/
 static gboolean
 menu_button_key_press_cb(GtkWidget *widget, 
                          GdkEventKey *event, gpointer data)
@@ -1911,6 +2062,7 @@ menu_button_key_press_cb(GtkWidget *widget,
 }
 
 
+/*ARGSUSED*/
 static void
 menu_pos_func(GtkMenu *menu, gint *x, gint *y,
               gboolean *push_in, gpointer user_data)
@@ -1948,6 +2100,36 @@ make_menu_button(gchar *label_text, int n)
 }
 
 
+static void
+toggle_expressions()
+{
+    /* TODO: Always do clear things when mode is changed. */
+
+    v->syntax = v->syntax ^ 1;
+    switch (v->syntax) {
+        case npa:
+            v->noparens = 0;
+            MPstr_to_num("0", DEC, v->MPdisp_val);
+            show_display(v->MPdisp_val);
+            update_statusbar(_("Activated no operator precedence mode"), "");
+            break;
+
+        case exprs:
+            v->e.calc_complete = 0;
+            MPstr_to_num("0", DEC, v->e.ans);
+            exp_del();
+            show_display(v->e.ans);
+            update_statusbar(
+                _("Activated expression mode with operator precedence"), "");
+            break;
+
+        default:
+            assert(0);
+    }
+    put_resource(R_SYNTAX, Rsstr[v->syntax]);
+}
+
+
 /* Handle menu bar menu selection. */
 
 static void
@@ -1970,7 +2152,7 @@ mb_proc(GtkAction *action)
         show_ascii_frame();
     } else if (EQUAL(name, "Memory")) {
         v->rstate = !v->rstate;
-        do_memory(v->rstate);
+        do_memory();
     } else if (EQUAL(name, "Contents")) {
         help_cb();
     } else if (EQUAL(name, "LSPlaces")) {
@@ -1981,10 +2163,13 @@ mb_proc(GtkAction *action)
         SSCANF(name,"RSPlaces%d", &choice);
         choice += (choice < 10) ? '0' : 'A' - 10;
         handle_menu_selection(X->mrec[(int) M_RSHF], choice);
+    } else if (EQUAL(name, "ArithmeticPrecedence")) {
+        toggle_expressions();
     }
 }
 
 
+/*ARGSUSED*/
 static void 
 mb_base_radio_proc(GtkAction *action, GtkRadioAction *current)
 {
@@ -2004,6 +2189,7 @@ mb_base_radio_proc(GtkAction *action, GtkRadioAction *current)
 }
 
 
+/*ARGSUSED*/
 static void
 mb_acc_radio_proc(GtkAction *action, GtkRadioAction *current)
 {
@@ -2013,10 +2199,15 @@ mb_acc_radio_proc(GtkAction *action, GtkRadioAction *current)
 	return;
     }
 
-    handle_menu_selection(X->mrec[(int) M_ACC], name[2]);
+    if (EQUAL(name, "SPOther")) {
+        show_precision_frame();
+    } else {
+        handle_menu_selection(X->mrec[(int) M_ACC], name[2]);
+    }
 }
 
 
+/*ARGSUSED*/
 static void
 mstz_proc(GtkAction *action)
 {
@@ -2082,7 +2273,6 @@ put_function(int n, char *fun_value, char *fun_name)
 }
 
 
-
 /* Put gcalctool resource into deskset database. */
 
 void
@@ -2096,6 +2286,7 @@ put_resource(enum res_type rtype, char *value)
 }
 
 
+/*ARGSUSED*/
 static void
 quit_cb(GtkWidget *widget, gpointer user_data)
 {
@@ -2126,6 +2317,59 @@ reset_mode_values(enum mode_type mtype)
 }
 
 
+/*ARGUSED*/
+static void
+spframe_adj_cb(GtkAdjustment *adj, gpointer data)
+{
+    g_print("New value is %f\n", adj->value);
+}
+
+
+/*ARGSUSED*/
+static void
+spframe_cancel_cb(GtkButton *button, gpointer user_data)
+{
+    gtk_widget_hide(X->spframe);
+}
+
+
+/*ARGSUSED*/
+static gboolean
+spframe_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+    if (event->keyval == GDK_minus) {
+        update_statusbar(_("Accuracy value out of range"),
+                         "gtk-dialog-error");
+        beep();
+    }
+
+    return(FALSE);
+}
+
+
+/*ARGSUSED*/
+static void
+spframe_ok_cb(GtkButton *button, gpointer user_data)
+{
+    char intval[5];
+    int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(X->spframe_val));
+
+    v->accuracy = val;
+
+    SPRINTF(intval, "%d", v->accuracy);
+    put_resource(R_ACCURACY, intval);
+
+    set_accuracy_menu_item(v->accuracy);
+    set_accuracy_tooltip(v->accuracy);
+    set_accuracy_toggle(v->accuracy);
+
+    make_registers();
+    refresh_display();
+
+    gtk_widget_hide(X->spframe);
+}
+
+
 static void
 set_button_state(GtkWidget *w, int isSensitive)
 {
@@ -2133,15 +2377,17 @@ set_button_state(GtkWidget *w, int isSensitive)
 }
 
 
-void
+static void
 set_accuracy_toggle(int val)
 {
     char name[MAXLINE];
     GtkWidget *acc;
 
-    SPRINTF(name, "/AccMenu/SP%c", val + '0');
-    acc = gtk_ui_manager_get_widget(X->ui, name);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(acc), TRUE);
+    if (val >= 0 && val <= 9) {
+        SPRINTF(name, "/AccMenu/SP%c", val + '0');
+        acc = gtk_ui_manager_get_widget(X->ui, name);
+        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(acc), TRUE);
+    }
 }
 
 
@@ -2205,6 +2451,32 @@ set_display(char *str, int minimize_changes)
 }
 
 
+void
+write_display(char *str)
+{
+    char localized[MAX_LOCALIZED];
+    gchar *text;
+    GtkTextBuffer *buffer;
+    GtkTextIter start, end;
+
+    if (str == NULL || *str == 0) str = " ";
+
+    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(X->display_item));
+    gtk_text_buffer_get_bounds(buffer, &start, &end);
+    text = gtk_text_buffer_get_text(buffer, &start, &end, TRUE);
+
+    gtk_text_buffer_delete(buffer, &start, &end);
+    
+    gtk_text_buffer_insert_with_tags_by_name(buffer,
+					     &end,
+					     str,
+					     -1,
+					     "x-large",
+					     NULL);
+    g_free(text);
+
+}
+
 #define SET_MENUBAR_ITEM_STATE(i, state) \
           g_object_set(gtk_ui_manager_get_action(X->ui, i),  \
 			"sensitive", state, NULL);
@@ -2258,6 +2530,8 @@ set_error_state(int error)
                            !v->error && (v->modetype == SCIENTIFIC)); 
     SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/Thousands",  !v->error); 
     SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/Memory",     !v->error); 
+    SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/ArithmeticPrecedence", !v->error); 
+    SET_MENUBAR_ITEM_STATE("/MenuBar/HelpMenu/About",      !v->error);
 }
 
 
@@ -2290,7 +2564,7 @@ set_mode(enum mode_type mode)
 {
     GtkRequisition *r;
     gint w, h;
-
+  
     switch (mode) {
         case BASIC:
             gtk_widget_hide(X->fin_panel);
@@ -2310,7 +2584,7 @@ set_mode(enum mode_type mode)
             gtk_widget_show(X->sci_panel);
             break;
     }
-
+  
     r = g_new0(GtkRequisition, 1);
     gtk_widget_size_request(X->menubar, r);
     w = r->width;
@@ -2318,31 +2592,34 @@ set_mode(enum mode_type mode)
     gtk_widget_size_request(X->display_item, r);
     w = MAX(w, r->width);
     h += r->height;
+
     if (GTK_WIDGET_VISIBLE(X->fin_panel)) {
-        gtk_widget_size_request(X->fin_panel, r);
-        w = MAX(w, r->width);
-        h += r->height;
+      gtk_widget_size_request(X->fin_panel, r);
+      w = MAX(w, r->width);
+      h += r->height;
     }
+
     if (GTK_WIDGET_VISIBLE(X->mode_panel)) {
-        gtk_widget_size_request(X->mode_panel, r);
-        w = MAX(w, r->width);
-        h += r->height;
+      gtk_widget_size_request(X->mode_panel, r);
+      w = MAX(w, r->width);
+          h += r->height;
     }
+
     if (GTK_WIDGET_VISIBLE(X->sci_panel)) {
-        gtk_widget_size_request(X->sci_panel, r);
-        w = MAX(w, r->width);
-    	h += r->height;
+      gtk_widget_size_request(X->sci_panel, r);
+      w = MAX(w, r->width);
+      h += r->height;
     }
-    
+  
     /* For initial display. */
     gtk_window_set_default_size(GTK_WINDOW(X->kframe), w, h);
     gtk_window_resize(GTK_WINDOW(X->kframe), w, h);
-    
+  
     g_free(r);
 }
 
 
-void
+static void
 set_item(enum item_type itemtype, int val)
 {
     if (!v->started) {
@@ -2420,7 +2697,7 @@ set_show_zeroes_toggle(int state)
 }
 
 
-void
+static void
 show_ascii_frame()      /* Display ASCII popup. */
 {
     if (X->aframe == NULL) {
@@ -2435,6 +2712,22 @@ show_ascii_frame()      /* Display ASCII popup. */
 }
 
 
+static void
+show_precision_frame()      /* Display Set Precision popup. */
+{
+    if (X->spframe == NULL) {
+        create_spframe();
+    }
+
+    if (gdk_window_is_visible(X->spframe->window) == FALSE) {
+        ds_position_popup(X->kframe, X->spframe, DS_POPUP_LEFT);
+    }
+    gtk_window_set_focus(GTK_WINDOW(X->spframe), GTK_WIDGET(X->spframe_val));
+    gtk_widget_show(X->spframe);
+}
+
+
+/*ARGSUSED*/
 static void
 trig_cb(GtkToggleButton *button, gpointer user_data)
 {
@@ -2449,11 +2742,13 @@ start_tool()
     set_item(BASEITEM, v->base);
     set_item(TTYPEITEM, v->ttype);
     set_item(NUMITEM, v->dtype);
+
     gtk_widget_show(X->kframe);
     gtk_main();
 }
 
 
+/*ARGSUSED*/
 static void
 ts_proc(GtkAction *action)
 {
