@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/mp.c,v 1.10 2004/02/17 17:25:44 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/mp.c,v 1.13 2004/07/27 22:06:31 richb Exp $
  *
- *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -935,27 +935,6 @@ L40:
 }
 
 
-void
-mpclr(int *x, int *n)
-{
-    int i__1;
-
-    static int i, i2, i3;
-
-/*  SETS X(T+3), ... , X(N+2) TO ZERO, USEFUL
- *  IF PRECISION IS GOING TO BE INCREASED.
- */
-
-    --x;                 /* Parameter adjustments */
-
-    if (*n <= MP.t) return;
-    i2 = MP.t + 3;
-    i3 = *n + 2;
-    i__1 = i3;
-    for (i = i2; i <= i__1; ++i) x[i] = 0;
-}
-
-
 void
 mpcmd(int *x, double *dz)
 {
